#!/bin/bash
# post_install script to populate the missing httpd.ssl entries
# when upgrading to v1.3.6+ from earlier version of SIP OPUS FW
# NB: Run this script only if the version is prior to 1.3.6!
# Created by Alex Simeonov, Barix AG, 02.09.2021

# first check if we are running on the device, and not on the production server
# The easiest is to check if there is UCI binary
# Note: use both logger and echo to see the messages both on syslog and console

function log_msg()
{
	logger $1
	echo $1
}

function is_device()
{
	uci_path=`which uci`
	if [ X"$uci_path" = X ]; then
		return 1
	else
		return 0 
	fi
}

# The busybox version of sort command does not support the -V option (version sort),
# so we are limited to compare the versions one digit at a time,
# assuming that the versions are in format x.y.z
# The function returns 0 for "true" (v1 < v2), or 1 for "false"
function is_old_version()
{
	v1="$1"
	v2="$2"
	if [ "$v1" == "$v2" ]; then
		return 1
	fi
	v1_maj=$(echo $v1 | cut -d'.' -f1)
	v1_min=$(echo $v1 | cut -d'.' -f2)
	v1_least=$(echo $v1 | cut -d'.' -f3)
	v2_maj=$(echo $v2 | cut -d'.' -f1)
	v2_min=$(echo $v2 | cut -d'.' -f2)
	v2_least=$(echo $v2 | cut -d'.' -f3)
	if [ "$v1_maj" -lt "$v2_maj" ]; then
		return 0
	else
		if [ "$v1_maj" -gt "$v2_maj" ]; then
			return 1
		else
			# v1_maj = v2_maj, check next digit
			if [ "$v1_min" -lt "$v2_min" ]; then
				return 0
			else
				if [ "$v1_min" -gt "$v2_min" ]; then
					return 1
				else
					# v1_min = v2_min, check next digit
					if [ "$v1_least" -lt "$v2_least" ]; then
						return 0
					else
						if [ "$v1_least" -gt "$v2_least" ]; then
							return 1
						else
							# no sense to check v1_least = v2_least, v1 = v2 is checked
							# at the very beginning, so must be v1_least < v2_least
							# case only remaining
							return 0
						fi
					fi
				fi
			fi
		fi
	fi
}

log_msg "Starting the post_install script after v1.3.6 or later FW update ..."

if ! is_device; then
		log_msg "No running device rootfs found, exiting ..."
		exit 0
fi

curr_ver=$(cat /barix/info/VERSION)
ref_ver="1.3.6"

# get the new roofs to be installed
# use jq -r to get rid of the enclosing double quotes in the result
cfg_dir=$(cat /etc/qiba-update-client.conf | jq -r .workSubDir)
rootfs_dir="/mnt/data/tmp/$cfg_dir/rootfs"

if is_old_version "$curr_ver" "$ref_ver"; then
	log_msg "Old FW version $curr_ver detected, merging the new HTTPd SSL defaults ..."
	# get new defaults. These are not in /barix/config...., but on the new partition to be flashed to SD card
	def_file="$rootfs_dir/barix/config/defaults/httpd"
	tmp_file="/tmp/uci.new"
	# add the new config section first
	echo "set httpd.ssl=section" > "$tmp_file"
	# export the new httpd.ssl.xxxxx settings to a temp UCI batch file
	cat "$def_file" | sed '/ssl/,$!d' | tail -n +2 | sed 's/^[[:blank:]]*//' | cut -d' ' -f2- | sed 's/ /=/' | xargs -I{} echo -e "set httpd.ssl.{}" >> "$tmp_file"
	# add them to current HTTPd settings
	uci batch -f "$tmp_file"
	uci commit
	# cleanup
	rm "$tmp_file"
	log_msg "HTTPd SSL default settings initialized!"
else
	log_msg "Current FW version already at the required minimum version $ref_ver, skipping HTTPd SSL default settings init ..."
fi


